@@ -9,14 +9,16 @@
 mod app;
 mod bank;
 mod contracts;
+pub mod custom_handler;
 pub mod error;
 mod executor;
 mod test_helpers;
 mod transactions;
 mod wasm;
 
-pub use crate::app::{next_block, App, Router};
+pub use crate::app::{next_block, App, AppBuilder, Router};
 pub use crate::bank::{Bank, BankKeeper};
 pub use crate::contracts::{Contract, ContractWrapper};
+pub use crate::custom_handler::CustomHandler;
 pub use crate::executor::{AppResponse, Executor};
 pub use crate::wasm::{parse_contract_addr, Wasm, WasmKeeper};
